@@ -1,8 +1,5 @@
-import json
-import time
 from ixnetwork_open_traffic_generator.timer import Timer
 from ixnetwork_open_traffic_generator.customfield import CustomField
-from ixnetwork_restpy import StatViewAssistant
 
 
 class TrafficItem(CustomField):
@@ -33,6 +30,7 @@ class TrafficItem(CustomField):
         'pfcPause': 'pfc_pause',
         'vlan': 'vlan',
         'ipv4': 'ipv4',
+        'ipv6': 'ipv6',
         'tcp': 'tcp',
         'udp': 'udp',
         'gtpu': 'gtpv1',
@@ -45,6 +43,7 @@ class TrafficItem(CustomField):
         'pfcpause': 'pfcPause',
         'vlan': 'vlan',
         'ipv4': 'ipv4',
+        'ipv6': 'ipv6',
         'tcp': 'tcp',
         'udp': 'udp',
         'gtpv1': 'gtpu',
@@ -107,6 +106,17 @@ class TrafficItem(CustomField):
         'dst': 'ipv4.header.dstIp',
     }
 
+    _IPV6 = {
+        'version' : 'ipv6.header.versionTrafficClassFlowLabel.version',
+        'traffic_class' : 'ipv6.header.versionTrafficClassFlowLabel.trafficClass',
+        'flow_label' : 'ipv6.header.versionTrafficClassFlowLabel.flowLabel',
+        'payload_length' : 'ipv6.header.payloadLength',
+        'next_header' : 'ipv6.header.nextHeader',
+        'hop_limit' : 'ipv6.header.hopLimit',
+        'src' : 'ipv6.header.srcIP',
+        'dst' : 'ipv6.header.dstIP'
+    }
+    
     _TOS = {
         "precedence": "ipv4.header.priority.tos.precedence",
         "delay": "ipv4.header.priority.tos.delay",
@@ -208,6 +218,7 @@ class TrafficItem(CustomField):
     def _configure_options(self, flow):
         enable_min_frame_size = False
         if (len(flow.packet) == 1 and flow.packet[0].choice == 'pfcpause'
+                and flow.size is not None
                 and flow.size.choice == 'fixed' and flow.size.fixed <= 64):
             enable_min_frame_size = True
         if self._api._traffic.EnableMinFrameSize != enable_min_frame_size:
@@ -332,15 +343,19 @@ class TrafficItem(CustomField):
                            field_type_id,
                            pattern,
                            field_choice=False):
-        if pattern == None:
-            return
-
+        
         custom_field = getattr(self, field_type_id, None)
         if custom_field is not None:
-            custom_field(ixn_field, pattern)
+            if pattern is not None:
+                custom_field(ixn_field, pattern)
             return
 
         ixn_field = ixn_field.find(FieldTypeId=field_type_id)
+        self._set_default(ixn_field, field_choice)
+        
+        if pattern is None:
+            return
+        
         if pattern.choice == 'fixed':
             ixn_field.update(Auto=False,
                              ActiveFieldChoice=field_choice,
@@ -379,6 +394,20 @@ class TrafficItem(CustomField):
             ixn_field.TrackingEnabled = True
             self._api.ixn_objects[pattern.ingress_result_name] = ixn_field.href
 
+    def _set_default(self, ixn_field, field_choice):
+        """We are setting all the field to default. Otherwise test is keeping the same value from previous run."""
+        if ixn_field.ReadOnly:
+            return
+        
+        if ixn_field.SupportsAuto:
+            if ixn_field.Auto is not True:
+                ixn_field.Auto = True
+        else:
+            ixn_field.update(Auto=False,
+                             ActiveFieldChoice=field_choice,
+                             ValueType='singleValue',
+                             SingleValue=ixn_field.DefaultValue)
+
     def _configure_size(self, ixn_stream, size):
         """ Transform frameSize flows.size to /traffic/trafficItem[*]/configElement[*]/frameSize
         """
@@ -462,8 +491,14 @@ class TrafficItem(CustomField):
         3) Execute requested transmit action (start|stop|pause|resume)
         """
         regex = ''
-        if request.flow_names is not None and len(request.flow_names) > 0:
-            regex = '^(%s)$' % '|'.join(request.flow_names)
+        flow_names = [flow.name for flow in self._api._config.flows]
+        if request and request.flow_names:
+            flow_names = request.flow_names
+        if len(flow_names) == 1:
+            regex = '^%s$' % flow_names[0]
+        elif len(flow_names) > 1:
+            regex = '^(%s)$' % '|'.join(flow_names)
+
         if request.state == 'start':
             all_flow_names = ' '.join(
                 [flow.name for flow in self._api.config.flows])
@@ -487,20 +522,20 @@ class TrafficItem(CustomField):
             with Timer(self._api, 'Flows start'):
                 self._api._traffic_item.StartStatelessTrafficBlocking()
         elif request.state == 'stop':
-            self._api._traffic_item.find(Name=regex, State='^start$')
+            self._api._traffic_item.find(Name=regex, State='^started$')
             if len(self._api._traffic_item) > 0:
                 with Timer(self._api, 'Flows stop'):
                     self._api._traffic_item.StopStatelessTrafficBlocking()
         elif request.state == 'pause':
-            self._api._traffic_item.find(Name=regex, State='^start$')
+            self._api._traffic_item.find(Name=regex, State='^started$')
             if len(self._api._traffic_item) > 0:
                 with Timer(self._api, 'Flows pause'):
-                    self._api._traffic_item.PauseStatelessTrafficBlocking(True)
+                    self._api._traffic_item.PauseStatelessTraffic(True)
         elif request.state == 'resume':
             self._api._traffic_item.find(Name=regex, State='^stopped$')
             if len(self._api._traffic_item) > 0:
                 with Timer(self._api, 'Flows resume'):
-                    self._api._traffic_item.PauseStatelessTrafficBlocking(False)
+                    self._api._traffic_item.PauseStatelessTraffic(False)
 
     def _set_result_value(self,
                           row,
@@ -570,7 +605,7 @@ class TrafficItem(CustomField):
                         self._set_result_value(
                             flow_row, external_name, 0, external_type
                         )
-                    flow_rows[flow_row['name'] + flow_row['port_tx'] + flow_row['port_rx']] = flow_row
+                    flow_rows[traffic_item['name'] + stream['txPortName'] + rx_port_name] = flow_row
 
         # resolve result values
         table = self._api.assistant.StatViewAssistant(
