@@ -86,7 +86,7 @@ class Vport(object):
     }
     _FLOW_CONTROL_MAP = {
         'ieee_802_1qbb': 'ieee802.1Qbb',
-        'ieee_802_3x': 'ieee_802_3x'
+        'ieee_802_3x': 'ieee802.3x'
     }
 
     _RESULT_COLUMNS = [
@@ -133,9 +133,12 @@ class Vport(object):
 
     def _import(self, imports):
         if len(imports) > 0:
-            for errata in self._resource_manager.ImportConfig(
-                    json.dumps(imports), False):
-                self._api.info(errata)
+            errata = self._resource_manager.ImportConfig(
+                json.dumps(imports), False)
+            for item in errata:
+                self._api.warning(item)
+            return len(errata) == 0
+        return True
 
     def _delete_vports(self):
         """Delete any vports from the api server that do not exist in the new config
@@ -249,7 +252,7 @@ class Vport(object):
         if len(check_addresses) > 0:
             with Timer(
                     self._api,
-                    'Location hosts check [%s]' % ', '.join(check_addresses)):
+                    'Location hosts ready [%s]' % ', '.join(check_addresses)):
                 start_time = time.time()
                 while True:
                     chassis.find(Hostname='^(%s)$' % '|'.join(check_addresses),
@@ -271,7 +274,7 @@ class Vport(object):
             location_supported = False
 
         locations = []
-        self._add_hosts(10)
+        self._add_hosts(60)
         vports = self._api.select_vports()
         imports = []
         clear_locations = []
@@ -311,17 +314,19 @@ class Vport(object):
             if len(self._api._vport) > 0:
                 self._api._vport.ConnectPorts()
             start = time.time()
-            timeout = 30
+            timeout = 10
             while True:
                 self._api._vport.find(Name='^(%s)$' % '|'.join(locations),
                                       ConnectionState='^connectedLink')
                 if len(self._api._vport) == len(locations):
                     break
                 if time.time() - start > timeout:
+                    unreachable = []
+                    self._api._vport.find(ConnectionState='^(?!connectedLink).*$')
+                    for vport in self._api._vport:
+                        unreachable.append('%s [%s: %s]' % (vport.Name, vport.ConnectionState, vport.ConnectionStatus))
                     raise RuntimeError(
-                        'After %s seconds, not all locations [%s] are reachable'
-                        % (timeout, ', '.join(
-                            [vport.Name for vport in self._api._vport])))
+                        'After %s seconds, %s are unreachable' % (timeout, ', '.join(unreachable)))
                 time.sleep(2)
             for vport in self._api._vport.find(
                     ConnectionState='^(?!connectedLinkUp).*$'):
@@ -337,17 +342,28 @@ class Vport(object):
             return
         if self._api.config.layer1 is None:
             return
-        vports = self._api.select_vports()
-        # set the vport type first and commit it otherwise some settings
-        # such as l1config/... will not be committed properly
-        imports = []
         reset_auto_negotiation = dict()
+        # set and commit the card resource mode
+        vports = self._api.select_vports()
+        imports = []
+        for layer1 in self._api.config.layer1:
+            for port_name in layer1.port_names:
+                self._set_card_resource_mode(vports[port_name], layer1,
+                                             imports)
+        if self._import(imports) is False:
+            # WARNING: this retry is because no reasonable answer as to why
+            # changing card mode periodically fails with this opaque message
+            # 'Releasing ownership on ports failed.'
+            self._api.info('Retrying card resource mode change')
+            self._import(imports)
+        # set the vport type
+        imports = []
         for layer1 in self._api.config.layer1:
             for port_name in layer1.port_names:
                 self._set_vport_type(vports[port_name], layer1, imports)
         self._import(imports)
-        # set the resource mode changes and any other l1config/... changes
-        # and commit them
+        vports = self._api.select_vports()
+        # set the remainder of l1config properties
         imports = []
         for layer1 in self._api.config.layer1:
             for port_name in layer1.port_names:
@@ -372,12 +388,18 @@ class Vport(object):
                 'connectedLinkUp', 'connectedLinkDown'
         ]:
             return
-        self._set_card_resource_mode(vport, layer1, imports)
+        self._set_fcoe(vport, layer1, imports)
+        self._import(imports)
+        
         self._set_auto_negotiation(vport, layer1, imports)
 
     def _set_card_resource_mode(self, vport, layer1, imports):
         """If the card has an aggregation mode set it according to the speed
         """
+        if vport['connectionState'] not in [
+                'connectedLinkUp', 'connectedLinkDown'
+        ]:
+            return
         speed_mode_map = {
             'speed_1_gbps': 'normal',
             'speed_10_gbps': 'tengig',
@@ -436,9 +458,10 @@ class Vport(object):
         if fcoe is False and vport_type.endswith('Fcoe'):
             vport_type = vport_type.replace('Fcoe', '')
         if vport_type != vport['type']:
-            imports.append({'xpath': vport['xpath'], 'type': vport_type})
-        if fcoe is True and vport_type.endswith('Fcoe'):
-            self._configure_fcoe(vport, layer1.flow_control, imports)
+            imports.append({
+                'xpath': vport['xpath'] + '/l1Config',
+                'currentType': vport_type
+            })
         return vport_type
 
     def _set_ethernet_auto_negotiation(self, vport, layer1, imports):
@@ -516,35 +539,33 @@ class Vport(object):
                 layer1.auto_negotiate,
             })
 
-    def _configure_fcoe(self, vport, flow_control, imports):
-        if flow_control is not None and flow_control.choice == 'ieee_802_1qbb':
-            pfc = flow_control.ieee_802_1qbb
-            fcoe = {
-                'xpath':
-                vport['xpath'] + '/l1Config/' +
-                vport['type'].replace('Fcoe', '') + '/fcoe',
-                'enablePFCPauseDelay':
-                False if pfc.pfc_delay == 0 else True,
-                'flowControlType':
-                Vport._FLOW_CONTROL_MAP[flow_control.choice],
-                'pfcPauseDelay':
-                pfc.pfc_delay,
-                'pfcPriorityGroups': [
-                    -1 if pfc.pfc_class_0 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_1 is None else pfc.pfc_class_1,
-                    -1 if pfc.pfc_class_2 is None else pfc.pfc_class_2,
-                    -1 if pfc.pfc_class_3 is None else pfc.pfc_class_3,
-                    -1 if pfc.pfc_class_4 is None else pfc.pfc_class_4,
-                    -1 if pfc.pfc_class_5 is None else pfc.pfc_class_5,
-                    -1 if pfc.pfc_class_6 is None else pfc.pfc_class_6,
-                    -1 if pfc.pfc_class_7 is None else pfc.pfc_class_7,
-                ],
-                'priorityGroupSize':
-                'priorityGroupSize-8',
-                'supportDataCenterMode':
-                True
-            }
-            imports.append(fcoe)
+    def _set_fcoe(self, vport, layer1, imports):
+        if hasattr(layer1, 'flow_control') and layer1.flow_control is None:
+            return
+        xpath = '%s/l1Config/%s/fcoe' % (vport['xpath'], vport['type'].replace(
+            'Fcoe', ''))
+        fcoe = {
+            'xpath': xpath,
+            'flowControlType':
+            Vport._FLOW_CONTROL_MAP[layer1.flow_control.choice]
+        }
+        if layer1.flow_control.choice == 'ieee_802_1qbb':
+            pfc = layer1.flow_control.ieee_802_1qbb
+            fcoe['enablePFCPauseDelay'] = False if pfc.pfc_delay == 0 else True
+            fcoe['pfcPauseDelay'] = pfc.pfc_delay
+            fcoe['pfcPriorityGroups'] = [
+                -1 if pfc.pfc_class_0 is None else pfc.pfc_class_0,
+                -1 if pfc.pfc_class_1 is None else pfc.pfc_class_1,
+                -1 if pfc.pfc_class_2 is None else pfc.pfc_class_2,
+                -1 if pfc.pfc_class_3 is None else pfc.pfc_class_3,
+                -1 if pfc.pfc_class_4 is None else pfc.pfc_class_4,
+                -1 if pfc.pfc_class_5 is None else pfc.pfc_class_5,
+                -1 if pfc.pfc_class_6 is None else pfc.pfc_class_6,
+                -1 if pfc.pfc_class_7 is None else pfc.pfc_class_7,
+            ]
+            fcoe['priorityGroupSize'] = 'priorityGroupSize-8'
+            fcoe['supportDataCenterMode'] = True
+        imports.append(fcoe)
 
     def _clear_ownership(self, locations):
         try:
@@ -598,8 +619,18 @@ class Vport(object):
             self._column_names = []
         else:
             self._column_names = request.column_names
+        
+        port_filter = {'property': 'name', 'regex': '.*'}
+        port_names = [port.name for port in self._api._config.ports]
+        if request and request.port_names:
+            port_names = request.port_names
+        if len(port_names) == 1:
+            port_filter['regex'] = '^%s$' % port_names[0]
+        elif len(port_names) > 1:
+            port_filter['regex'] = '^(%s)$' % '|'.join(port_names)
+            
         port_rows = {}
-        for vport in self._api.select_vports().values():
+        for vport in self._api.select_vports(port_name_filters=[port_filter]).values():
             port_row = {}
             self._set_result_value(port_row, 'name', vport['name'])
             location = vport['location']
